﻿


'
'
'/**
'--------------------------------------------------
' Version .....: 18/08/2010
'               [dd/mm/aaaa] 
' Original en .: Util
'--------------------------------------------------
'
'
' -------------------------------------------------
' Clase ....: JulianaAstronomica
' Lenguaje .: (.NET Framework 3.5) - Visual Basic .NET (Version 9.0) - 2009
' Autor ....:  - Aguila - Joaquin
' Emilio ...: mailto:joaquin@medina.name
'
' DESCRIPCION.:
'     Realiza los cálculos para convertir una fecha del calendario Gregoriano 
'     en una fecha juliana astronómica y viceversa
'
' OBSERVACIONES.:
'    *  Sin la referecia [Microsoft.VisualBasic] Sin Usar ControlChars
'    *  Los cálculos no sé porque se hacen pero funcionan
'    *  Las funciones de transformación están adaptadas de unas funciones  
'       escritas en Java Script localizadas en la siguiente Web:
'        http://www.imcce.fr/en/grandpublic/temps/jour_julien.php
'
' BIBLIOGRAFIA.:
'        http://www.imcce.fr/en/grandpublic/temps/jour_julien.php
'        http://es.wikipedia.org/wiki/Fecha_juliana
'
' CULTURILLA.: 
' * En el 1582, José Scaliger de Leyden fijó una escala continua de tiempo fijando su 
'   origen en el 1 de enero del año 4713 a. C. a las 12h del mediodía  
'   (en esta época el día empezaba a mediodía y no como es costumbre ahora, en que el  
'   día comienza a medianoche) y contando los días solares correlativamente.  
'   Este número se llama fecha juliana. 
' * La fecha juliana (JD) es una cuenta continua de días y fracciones de estos  
'   contados desde el mismo período inicial. 
' * Por ejemplo: El día 11 de julio de 1997 a las 12 h TU se completa el día 2.450.641
' * Por ejemplo: la fecha Juliana 2451545.0 TT (Tiempo Terrestre), o  
'   1 de enero de 2000, mediodía TT. Es equivalente al 1 de enero de 2000. 
' * La parte fraccional da la hora del día desde el amanecer UT como una  
'   fracción decimal de un día en el que 0,5 sería la medianoche UT. 
' * Esta fecha es usada ampliamente para indicar un instante en el tiempo estándar  
'   para la medición de las posiciones de los cuerpos celestes y otros eventos estelares.  
'   Por ejemplo, aunque imperceptible a simple vista, las estrellas se mueven en el espacio,  
'   y es necesario, para describir su posición en el firmamento, especificar la fecha a  
'   la que se refiere dicha posición.
'
'
' MODIFICACIONES [dd/mm/aaaa] 
' - 18/08/2010 - Creación
' -------------------------------------------------
'
'*/
'

' -------------------------------------------------
Option Explicit On
Option Strict On
Option Infer Off ' Deshabilita la inferencia de tipo de variable local.
Option Compare Binary




Namespace Util.Fechas

    ''' <summary>
    '''    Realiza los cálculos para convertir una fecha del calendario Gregoriano en una fecha juliana astronómica y viceversa
    ''' </summary>
    ''' <remarks>
    ''' </remarks>
    Public Class JulianaAstronomica

#Region "Estructura para devolver los datos obtenidos"

        ''' <summary>
        '''  Contiene los datos (Datos iniciales y resultado transformado) de transformar una fecha gregoriana en juliana astronómica y viceversa
        ''' </summary>
        ''' <remarks>
        ''' Solo hay datos, no se hacen cálculos
        ''' Se emplea por las funciones que hacen los cálculos de transformación para devolver los resultados 
        ''' </remarks>
        Public Structure JulianaAstronomicaDatos

#Region "Propiedades"


            Private _Año As Integer
            Private _Mes As Integer
            Private _Dia As Integer
            Private _Hora As Integer
            Private _Minutos As Integer
            Private _Segundos As Integer
            Private _JulianaAstronomica As Decimal


            ''' <summary>
            '''    Devuelve el año de la fecha tratada
            ''' </summary>
            Public ReadOnly Property Año As Integer
                Get
                    Return _Año
                End Get
            End Property
            ''' <summary>
            '''    Devuelve el mes de la fecha tratada
            ''' </summary>
            Public ReadOnly Property Mes As Integer
                Get
                    Return _Mes
                End Get
            End Property
            ''' <summary>
            '''    Devuelve el dia de la fecha tratada
            ''' </summary>
            Public ReadOnly Property Dia As Integer
                Get
                    Return _Dia
                End Get
            End Property
            ''' <summary>
            '''    Devuelve la hora de la fecha tratada
            ''' </summary>
            Public ReadOnly Property Hora As Integer
                Get
                    Return _Hora
                End Get
            End Property
            ''' <summary>
            '''    Devuelve los minutos de la fecha tratada
            ''' </summary>
            Public ReadOnly Property Minutos As Integer
                Get
                    Return _Minutos
                End Get
            End Property
            ''' <summary>
            '''    Devuelve los segundos de la fecha tratada
            ''' </summary>
            Public ReadOnly Property Segundos As Integer
                Get
                    Return _Segundos
                End Get
            End Property
            ''' <summary>
            '''    Devuelve la fecha en formato Juliano astronómico
            ''' </summary>
            Public ReadOnly Property JulianaAstronomica As Decimal
                Get
                    Return _JulianaAstronomica
                End Get
            End Property

#End Region


#Region "To String"


            ''' <summary>
            '''    Funcion de trabajo.
            '''    Formatea la fecha en una cadena segun el estandar [ISO 8601:2004]  
            '''    'Calendar Date extendido' (Ejemplo 2010-08-25) o 
            '''    'Calendar Date basico'    (Ejemplo 20100825)
            ''' </summary>
            ''' <param name="extendida">
            '''    Valor 'true' indica hay que formatear la fecha en formato extendido, 
            '''    Valor false en formato basico
            ''' </param>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Private Function AuxCalendarDateToString(ByVal extendida As Boolean) As String


                Dim signoNegativo As Boolean = False
                If Año < 1 Then signoNegativo = True

                Dim añoAux As String = (Math.Abs(Año)).ToString(System.Globalization.CultureInfo.CurrentCulture)
                Dim mesAux As String = Mes.ToString(System.Globalization.CultureInfo.CurrentCulture)
                Dim diaAux As String = Dia.ToString(System.Globalization.CultureInfo.CurrentCulture)

                Do While añoAux.Length < 4
                    añoAux = "0" & añoAux
                Loop
                If signoNegativo = True Then
                    añoAux = "-" & añoAux
                End If


                Do While mesAux.Length < 2
                    mesAux = "0" & mesAux
                Loop
                Do While diaAux.Length < 2
                    diaAux = "0" & diaAux
                Loop

                If extendida = True Then
                    Return añoAux & "-" & mesAux & "-" & diaAux
                Else
                    Return añoAux & mesAux & diaAux
                End If

            End Function


            ''' <summary>
            '''    Devuelve una cadena con una fecha formateada segun el estandar [ISO 8601:2004]  'Calendar Date extendido' (Ejemplo 2010-08-25)
            ''' </summary>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Public ReadOnly Property ToFechaISO8601CalendarDateExtendida As String
                Get
                    Return AuxCalendarDateToString(True)
                End Get
            End Property

            ''' <summary>
            '''   Devuelve una cadena con una fecha formateada segun el estandar [ISO 8601:2004] 'Calendar Date basico' (Ejemplo 20100825)
            ''' </summary>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Public ReadOnly Property ToFechaISO8601CalendarDateBasica As String
                Get
                    Return AuxCalendarDateToString(False)
                End Get
            End Property


            ''' <summary>
            '''    Funcion de trabajo.
            '''    Formatea la Hora en una cadena segun el estandar [ISO 8601:2004]  
            '''    'Local time Extendido' (Ejemplo 23:41:57)
            '''    'Local time basico'    (Ejemplo 234157)
            ''' </summary>
            ''' <param name="extendida">
            '''    Valor 'true' indica hay que formatear la hora en formato extendido, 
            '''    Valor false en formato basico
            ''' </param>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Private Function AuxLocalTimeToString(ByVal extendida As Boolean) As String

                Dim horaAux As String = Me.Hora.ToString(System.Globalization.CultureInfo.CurrentCulture)
                Dim minutoAux As String = Me.Minutos.ToString(System.Globalization.CultureInfo.CurrentCulture)
                Dim segundoAux As String = Me.Segundos.ToString(System.Globalization.CultureInfo.CurrentCulture)

                Do While horaAux.Length < 2
                    horaAux = "0" & horaAux
                Loop
                Do While minutoAux.Length < 2
                    minutoAux = "0" & minutoAux
                Loop
                Do While segundoAux.Length < 2
                    segundoAux = "0" & segundoAux
                Loop

                If extendida = True Then
                    Return horaAux & ":" & minutoAux & ":" & segundoAux
                Else
                    Return horaAux & minutoAux & segundoAux
                End If

            End Function


            ''' <summary>
            '''    Devuelve una cadena con una hora formateada segun el estandar [ISO 8601:2004] 'Local time Extendido' (Ejemplo 23:41:57)
            ''' </summary>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Public ReadOnly Property ToHoraISO8601LocalTimeExtendida As String
                Get
                    Return AuxLocalTimeToString(True)
                End Get
            End Property

            ''' <summary>
            '''    Devuelve una cadena con una hora formateada segun el estandar [ISO 8601:2004] 'Local time basico' (Ejemplo 234157)
            ''' </summary>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Public ReadOnly Property ToHoraISO8601LocalTimeBasica As String
                Get
                    Return AuxLocalTimeToString(False)
                End Get
            End Property



            ''' <summary>
            '''  Devuelve una cadena con una fecha y una hora formateada segun el estandar [ISO 8601:2004] 'calendar date and local time extendido' (Ejemplo 2010-08-25T23:41:57)
            ''' </summary>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Public ReadOnly Property ToFechaHoraISO8601GregorianaExtendida As String
                Get
                    Return ToFechaISO8601CalendarDateExtendida & "T" & ToHoraISO8601LocalTimeExtendida
                End Get
            End Property

            ''' <summary>
            '''  Devuelve una cadena con una fecha y una hora formateada segun el estandar [ISO 8601:2004] 'calendar date and local time basico' (Ejemplo 20100825T234157)
            ''' </summary>
            ''' <remarks> La clase trabaja con fechas del calendario gregoriano</remarks>
            Public ReadOnly Property ToFechaHoraISO8601GregorianaBasica As String
                Get
                    Return ToFechaISO8601CalendarDateBasica & "T" & ToHoraISO8601LocalTimeBasica
                End Get
            End Property


            ''' <summary>
            '''    Devuelve una cadena con la fecha en formato Juliano astronómico ( ejemplo )
            ''' </summary>
            Public ReadOnly Property JulianaAstronomicaToString As String
                Get
                    Return _JulianaAstronomica.ToString(System.Globalization.CultureInfo.CurrentCulture)
                End Get
            End Property


#End Region


#Region "Constructores"

            ''' <summary>
            '''   Constructor de la estructura
            ''' </summary>
            ''' <param name="YY"> Fecha Gregoriana: El año </param>
            ''' <param name="MM"> Fecha Gregoriana: El mes</param>
            ''' <param name="DD"> Fecha Gregoriana: El dia</param>
            ''' <param name="HR"> Fecha Gregoriana: La hora</param>
            ''' <param name="Min">Fecha Gregoriana: Los minutos</param>
            ''' <param name="SS"> Fecha Gregoriana: Los segundos</param>
            ''' <param name="fechaJulianaAstronomica">El numero que representa la fecha Juliana astronomica</param>
            ''' <remarks></remarks>
            Public Sub New( _
                           ByVal YY As Integer, ByVal MM As Integer, ByVal DD As Integer, _
                           ByVal HR As Integer, ByVal Min As Integer, ByVal SS As Integer, _
                           ByVal fechaJulianaAstronomica As Decimal)

                _Año = YY
                _Mes = MM
                _Dia = DD
                _Hora = HR
                _Minutos = Min
                _Segundos = SS
                _JulianaAstronomica = fechaJulianaAstronomica

            End Sub

#End Region

        End Structure
#End Region

        Public Sub New()
            ' no hacer nada
        End Sub

#Region " Calculos GregorianoToJulianoAstronomico"


#Region "Documentacion"


        ' funcion  original en JavaScript copiada de la Web
        ' http://www.imcce.fr/en/grandpublic/temps/jour_julien.php
        'function compute(form) {
        '    MM=(form.nmonth.value=="")? "0" : eval(form.nmonth.value);
        '    DD=(form.nday.value=="")? "0": eval(form.nday.value);
        '    YY=(form.nyear.value=="") ? "0" :eval(form.nyear.value);
        '    HR=(form.nhour.value=="")? "0" :eval(form.nhour.value);
        '    MN=(form.nminute.value=="") ? "0" :eval(form.nminute.value);
        '    SS=(form.nsecondes.value=="") ? "0" : eval(form.nsecondes.value);
        '    with (Math) {  
        '      HR = HR + (MN / 60) + (SS / 3600);
        '      GGG = 1;
        '      if( YY < 1582 ) GGG = 0;
        '      if( YY <= 1582 && MM < 10 ) GGG = 0;
        '      if( YY <= 1582 && MM == 10 && DD < 5 ) GGG = 0;
        '      JD = -1 * floor(7 * (floor((MM + 9) / 12) + YY) / 4);
        '      S = 1;
        '      if ((MM - 9)<0) S=-1;
        '      A = abs(MM - 9);
        '      J1 = floor(YY + S * floor(A / 7));
        '      J1 = -1 * floor((floor(J1 / 100) + 1) * 3 / 4);
        '      JD = JD + floor(275 * MM / 9) + DD + (GGG * J1);
        '      JD = JD + 1721027 + 2 * GGG + 367 * YY - 0.5;
        '      JD = JD + (HR / 24);
        '    }
        '    form.result.value = JD;
        '}
#End Region


        ''' <summary>
        '''    Calcula la fecha Juliana Astronomica correspondiente a la 
        '''    fecha del calendario Gregoriano pasada en los parametros. 
        '''    Carga los valores de las variables miembros de la clase.
        ''' </summary>
        ''' <param name="fechayhora">una fecha y hora en frmato [Date] </param>
        ''' <remarks> Utiliza la funcion que hace el trabajo para realizar lso calculos</remarks>
        ''' <returns> 
        '''      Una estructura [JulianaAstronomicaDatos] con los datos, tanto los 
        '''      iniciales del calculo como el resultado de los mismos
        ''' </returns>
        Public Overloads Shared Function GregorianoToJulianoAstronomico( _
                         ByVal fechayhora As Date) _
                         As JulianaAstronomicaDatos
            Return GregorianoToJulianoAstronomico( _
                        fechayhora.Year, fechayhora.Month, fechayhora.Day, _
                        fechayhora.Hour, fechayhora.Minute, fechayhora.Second)
        End Function

        ''' <summary>
        '''    Calcula la fecha Juliana Astronomica correspondiente a la 
        '''    fecha del calendario Gregoriano pasada en los parametros. 
        '''    Carga los valores de las variables miembros de la clase.
        ''' </summary>
        ''' <param name="YY"> El año </param>
        ''' <param name="MM"> El mes</param>
        ''' <param name="DD"> El dia</param>
        ''' <param name="HR"> La hora</param>
        ''' <param name="Min">Los minutos</param>
        ''' <param name="SS"> Los segundos</param>
        ''' <remarks> Utiliza la funcion que hace el trabajo para realizar lso calculos</remarks>
        ''' <returns> 
        '''      Una estructura [JulianaAstronomicaDatos] con los datos, tanto los 
        '''      iniciales del calculo como el resultado de los mismos
        ''' </returns>
        Public Overloads Shared Function GregorianoToJulianoAstronomico( _
                        ByVal YY As String, ByVal MM As String, ByVal DD As String, _
                        ByVal HR As String, ByVal Min As String, ByVal SS As String) _
                        As JulianaAstronomicaDatos

            '-----------------------------
            If YY = String.Empty Then YY = _
                   Date.Now.Year.ToString(System.Globalization.CultureInfo.CurrentCulture)
            Dim auxAño As Integer
            Try
                auxAño = Integer.Parse(YY)
            Catch ex As Exception
                auxAño = -4712
            End Try

            If MM = String.Empty Then MM = "1"
            Dim auxMes As Integer
            Try
                auxMes = Integer.Parse(MM)
            Catch ex As Exception
                auxMes = 1
            End Try

            If DD = String.Empty Then DD = "1"
            Dim auxDia As Integer
            Try
                auxDia = Integer.Parse(DD)
            Catch ex As Exception
                auxDia = 1
            End Try


            '-----------------------------
            If HR = String.Empty Then HR = "0"
            Dim auxHora As Integer
            Try
                auxHora = Integer.Parse(HR)
            Catch ex As Exception
                auxHora = 0
            End Try

            If Min = String.Empty Then Min = "0"
            Dim auxMin As Integer
            Try
                auxMin = Integer.Parse(Min)
            Catch ex As Exception
                auxMin = 0
            End Try

            If SS = String.Empty Then SS = "0"
            Dim auxSec As Integer
            Try
                auxSec = Integer.Parse(SS)
            Catch ex As Exception
                auxSec = 0
            End Try

            '-----------------------------
            Return GregorianoToJulianoAstronomico(auxAño, auxMes, auxDia, auxHora, auxMin, auxSec)

        End Function


        ''' <summary>
        '''    Calcula la fecha Juliana Astronomica correspondiente a la 
        '''    fecha del calendario Gregoriano pasada en los parametros. 
        '''    Carga los valores de las variables miembros de la clase.
        ''' </summary>
        ''' <param name="YY"> El año </param>
        ''' <param name="MM"> El mes</param>
        ''' <param name="DD"> El dia</param>
        ''' <param name="HR"> La hora</param>
        ''' <param name="Min">Los minutos</param>
        ''' <param name="SS"> Los segundos</param>
        ''' <remarks>       
        ''' <para>Esta funcion es la que hace el trabajo y realiza los calculos</para>
        ''' <para>Calcula un numero (decimal) con el valor Gregoriano Astronomico de la 
        '''       fecha recibida. Ejemplo. 2000-01-01T12:00:00 es 2451545,0 </para>
        ''' <para>Los resultados se cargan en las variables miembros de la clase</para>
        ''' <code>  
        '''    <para>No se porque funciona   </para>   
        '''    <para>Está copiada de una funcion en JavaScript encontrada en la pagina</para>
        '''    <para>http://www.imcce.fr/en/grandpublic/temps/jour_julien.php</para>
        ''' </code>
        ''' </remarks>
        ''' <returns> 
        '''      Una estructura [JulianaAstronomicaDatos] con los datos, tanto los 
        '''      iniciales del calculo como el resultado de los mismos
        ''' </returns>
        '''   <autor> Joaquin medina Serrano: joaquin@medina.name</autor>
        '''   <rights> Licencia Creative Commons Compartir-Igual 3.0</rights>
        '''   <Creacion>2010/08/18T18:19:00</Creacion>
        Public Overloads Shared Function GregorianoToJulianoAstronomico( _
                         ByVal YY As Integer, ByVal MM As Integer, ByVal DD As Integer, _
                         ByVal HR As Integer, ByVal Min As Integer, ByVal SS As Integer) _
                         As JulianaAstronomicaDatos


            Dim hora As Decimal = Convert.ToDecimal(HR + (Min / 60) + (SS / 3600))
            Dim GGG As Integer = 1

            If (YY < 1582) Then GGG = 0
            If (YY <= 1582 And MM < 10) Then GGG = 0
            If (YY <= 1582 And MM = 10 And DD < 5) Then GGG = 0

            Dim JD As Decimal = Convert.ToDecimal(-1 * Math.Floor(7 * (Math.Floor((MM + 9) / 12) + YY) / 4))
            Dim S As Integer = 1
            If ((MM - 9) < 0) Then S = -1
            Dim A As Integer = Math.Abs(MM - 9)

            Dim J1 As Integer = Convert.ToInt32(Math.Floor(YY + (S * Math.Floor(A / 7))))
            J1 = Convert.ToInt32(-1 * Math.Floor((Math.Floor(J1 / 100) + 1) * 3 / 4))
            JD = Convert.ToDecimal(JD + Math.Floor((275 * MM) / 9) + DD + (GGG * J1))
            JD = Convert.ToDecimal(JD + 1721027 + (2 * GGG) + (367 * YY) - 0.5)
            JD = Convert.ToDecimal(JD + (hora / 24))

            ' ---------------------------------------
            ' Resultados a la estructura
            Dim resultado As New JulianaAstronomicaDatos(YY, MM, DD, HR, Min, SS, JD)
            Return resultado
            ' ---------------------------------------
        End Function

#End Region

#Region " Calculos JulianoAstronomicoToGregoriano"



#Region "Documentacion"


        ' funcion  original en JavaScript copiada de la Web
        ' http://www.imcce.fr/en/grandpublic/temps/jour_julien.php
        'function reverse(form) {
        '            JD = eval(form.result.value)
        '   with (Math) {  
        '   	Z = floor(JD+0.5);
        '   	F = JD+0.5 - Z;
        '   	if (Z < 2299161) {
        '                    A = Z
        '		} else
        '      	{I = floor((Z - 1867216.25)/36524.25);
        '      	A = Z + 1 + I - floor(I/4);
        '   	 }
        '   	B = A + 1524;
        '   	C = floor((B - 122.1)/365.25);
        '   	D = floor(365.25 * C);
        '   	T = floor((B - D)/ 30.6001);
        '   	RJ = B - D - floor(30.6001 * T) + F;
        '   	JJ = floor(RJ);
        '   	RH = (RJ - floor(RJ)) * 24;
        '   	Heure = floor(RH);
        '   	Mn = floor((RH - Heure )*60);
        '   	Sec = ((RH - Heure )*60 - Mn )*60; 
        '   	if (T < 14) {
        '                        MM = T - 1
        '   	} else {
        '   	  if ((T == 14) || (T == 15))  MM = T - 13 
        '   	} 
        '   	if (MM > 2) {
        '                                AA = C - 4716
        '   	} else {
        '      	if ((MM == 1) || (MM == 2)) AA = C - 4715
        '   	}
        '   }

        ' form.nmonth.value =  MM;
        'form.nday.value   =  JJ;
        'form.nhour.value  =  Heure;
        'form.nyear.value  =  AA;
        'form.nminute.value=  Mn;
        'form.nsecondes.value=Sec;
#End Region


        ''' <summary>
        '''    Calcula la fecha del calendario Gregoriano correspondiente a la 
        '''    fecha Juliana Astronomica pasada en los parametros. 
        ''' </summary>
        ''' <param name="FechaJulianaAstronomica">
        '''        La fecha Juliana astronomica de la que queremos conocer 
        '''        su equivalente en el calendario Gregoriano
        ''' </param>
        ''' <returns> 
        '''      Una estructura [JulianaAstronomicaDatos] con los datos, tanto los 
        '''      iniciales del calculo como el resultado de los mismos
        ''' </returns>
        ''' <remarks>       
        ''' <para> Esta funcion es la que hace el trabajo y realiza los calculos</para>
        ''' <para> Calcula la fecha del calendario Gregoriano correspondiente un numero 
        '''        (decimal) con el valor Gregoriano Astronomico paraso por parametro
        '''        Ejemplo. la fecha Juliana astronomica [2451545,0]  se corresponde con 
        '''        la fecha del calendario Gregoriano siguiente [2000-01-01T12:00:00]
        ''' </para>
        ''' <code>  
        '''    <para>No se porque funciona </para>   
        '''    <para>Está copiada de una funcion en JavaScript encontrada en la pagina</para>
        '''    <para>http://www.imcce.fr/en/grandpublic/temps/jour_julien.php</para>
        ''' </code>
        ''' </remarks>
        '''   <autor> Joaquin medina Serrano: joaquin@medina.name</autor>
        '''   <rights> Licencia Creative Commons Compartir-Igual 3.0</rights>
        '''   <Creacion>2010/08/18T18:19:00</Creacion>
        Public Shared Function JulianoAstronomicoToGregoriano(ByVal FechaJulianaAstronomica As Decimal) As JulianaAstronomicaDatos

            Dim JD As Decimal = FechaJulianaAstronomica

            Dim Z As Integer = Convert.ToInt32(Math.Floor(JD + 0.5)) ' tiene la fecha
            Dim F As Decimal = Convert.ToDecimal(JD + 0.5 - Z) ' tiene la hora

            Dim I As Decimal = 0
            Dim A As Decimal = 0
            If (Z < 2299161) Then
                A = Z
            Else
                I = Convert.ToDecimal(Math.Floor((Z - 1867216.25) / 36524.25))
                A = Z + 1 + I - Math.Floor(I / 4)
            End If

            ' fecha
            Dim B As Decimal = A + 1524
            Dim C As Decimal = Convert.ToDecimal(Math.Floor((B - 122.1) / 365.25))   ' aqui
            Dim D As Decimal = Convert.ToDecimal(Math.Floor(365.25 * C))
            Dim T As Decimal = Convert.ToDecimal(Math.Floor((B - D) / 30.6001))

            ' Hora
            Dim RJ As Decimal = Convert.ToDecimal(B - D - Math.Floor(30.6001 * T) + F)
            Dim JJ As Decimal = Math.Floor(RJ)
            Dim RH As Decimal = (RJ - Math.Floor(RJ)) * 24
            Dim Heure As Decimal = Math.Floor(RH)
            Dim Mn As Integer = Convert.ToInt32(Math.Floor((RH - Heure) * 60))
            Dim Sec As Integer = Convert.ToInt32(((RH - Heure) * 60 - Mn) * 60)


            Dim MM As Integer = 0
            If (T < 14) Then
                MM = Convert.ToInt32(T - 1)
            Else
                If ((T = 14) Or (T = 15)) Then MM = Convert.ToInt32(T - 13)
            End If

            Dim AA As Integer = 0
            If (MM > 2) Then
                AA = Convert.ToInt32(C - 4716)
            Else
                If ((MM = 1) Or (MM = 2)) Then AA = Convert.ToInt32(C - 4715)
            End If

            ' ---------------------------------------
            ' Resultados a las variables de la clase
            '
            '_Año = AA
            '_Mes = MM
            '_Dia = CInt(JJ)
            '_Hora = CInt(Heure)
            '_Minutos = Mn
            '_Segundos = Sec
            '_JulianaAstronomica = JD
            ' ---------------------------------------
            ' ---------------------------------------
            ' Resultados a la estructura
            Dim resultado As New JulianaAstronomicaDatos(AA, MM, Convert.ToInt32(JJ), Convert.ToInt32(Heure), Mn, Sec, JD)
            Return resultado
            ' ---------------------------------------
        End Function
#End Region
    End Class
End Namespace